#ifndef _BLUEIO_CMDS_H_
#define _BLUEIO_CMDS_H_

/***************************************************************************************************/
//
// This file contains declarations for parameter and response structures used by the function 
// BlueIO_SendCmdAndGetResponse().
//
/***************************************************************************************************/

//
// General BLUEIO commands include:
//
#define BLUEIO_CMD_ID_GET_STATUS 0x10
#define BLUEIO_CMD_ID_START_MEASUREMENTS 0x18
#define BLUEIO_CMD_ID_STOP_MEASUREMENTS 0x19
#define BLUEIO_CMD_ID_INIT 0x1A
#define BLUEIO_CMD_ID_SET_MEASUREMENT_PERIOD 0x1B
#define BLUEIO_CMD_ID_GET_MEASUREMENT_PERIOD 0x1C
#define BLUEIO_CMD_ID_SET_LED_STATE 0x1D
#define BLUEIO_CMD_ID_GET_LED_STATE 0x1E
#define BLUEIO_CMD_ID_GET_SERIAL_NUMBER 0x20

#define BLUEIO_CMD_ID_WRITE_NV_MEM 0x21	
#define BLUEIO_CMD_ID_READ_NV_MEM 0x27
#define BLUEIO_CMD_ID_SET_SENSOR_CHANNEL_ENABLE_MASK 0x2C
#define BLUEIO_CMD_ID_GET_SENSOR_CHANNEL_ENABLE_MASK 0x2D
#define BLUEIO_CMD_ID_SET_COLLECTION_PARAMS 0x2E
#define BLUEIO_CMD_ID_GET_COLLECTION_PARAMS 0x2F

#define BLUEIO_CMD_ID_GET_ARCHIVED_RUN_IDS 0x30
#define BLUEIO_CMD_ID_DELETE_ARCHIVED_RUNS 0x31
#define BLUEIO_CMD_ID_SET_DAC_LEVEL 0x32
#define BLUEIO_CMD_ID_GET_DAC_LEVEL 0x33
#define BLUEIO_CMD_ID_CLEAR_ERROR_FLAGS 0x34

#define FIRST_BLUEIO_CMD_ID BLUEIO_CMD_ID_GET_STATUS
#define LAST_BLUEIO_CMD_ID BLUEIO_CMD_ID_CLEAR_ERROR_FLAGS

//Note that additional commands may be defined that are specific to individual types of sources.
//See WDSSCmds.h .
//
/***************************************************************************************************/
//

// Make sure structures get packed on 1-byte boundaries.  
// These preprocessor commands are in a few places throughout the code so if you change one, change them all!
#if defined TARGET_OS_WIN
	#pragma pack(push)
	#pragma pack(1)
#elif (defined TARGET_OS_MAC) || (defined TARGET_OS_LINUX)
	#pragma pack(1)
#endif

// The structures below define the parameter and response blocks associated with the commands defined above.
// Pointers to these structures are passed in to SendCmdAndGetResponse().
// If no parameter structure is defined for a command, then the command does not take parameters.
// If no response structure is defined for a command, then the only response associated with the command is BlueIODefaultCmdResponse.

#define BLUEIO_STATUS_SUCCESS 0
#define BLUEIO_STATUS_NOT_READY_FOR_NEW_CMD 0x30
#define BLUEIO_STATUS_CMD_NOT_SUPPORTED 0x31
#define BLUEIO_STATUS_INTERNAL_ERROR1 0x32
#define BLUEIO_STATUS_INTERNAL_ERROR2 0x33
#define BLUEIO_STATUS_ERROR_CANNOT_CHANGE_PERIOD_WHILE_COLLECTING 0x34
#define BLUEIO_STATUS_ERROR_CANNOT_READ_NV_MEM_BLK_WHILE_COLLECTING_FAST 0x35
#define BLUEIO_STATUS_ERROR_INVALID_PARAMETER 0x36
#define BLUEIO_STATUS_ERROR_CANNOT_WRITE_FLASH_WHILE_COLLECTING 0x37
#define BLUEIO_STATUS_ERROR_CANNOT_WRITE_FLASH_WHILE_HOST_FIFO_BUSY 0x38
#define BLUEIO_STATUS_ERROR_OP_BLOCKED_WHILE_COLLECTING 0x39
#define BLUEIO_STATUS_ERROR_CALCULATOR_CANNOT_MEASURE_WITH_NO_BATTERIES 0x3A

//If SendCmdAndGetResponse() returns kResponse_Error and (1 == *pnRespBytes), then *pRespBuf contains
//BlueIODefaultCmdResponse, even if a different response structure is defined for the command.
//Note that if you only need to know whether or not SendCmdAndGetResponse() fails, but you do not care about why,
//then you can always set the pRespBuf paramter to NULL.
typedef struct
{
	unsigned char status;//BLUEIO_STATUS_...
} BlueIODefaultCmdResponse;

/***************************************************************************************************/
//Parameter block for the BLUEIO_CMD_ID_START_MEASUREMENTS command:
//This parameter block is optional for BLUEIO_CMD_ID_START_MEASUREMENTS. If the SendCmdAndGetResponse
//parameter block ptr is NULL, then the device should use defaults. Typically, this means real time
//data collection is triggered immediately.

typedef struct
{
	unsigned char lsbyteLswordDataRunId;//0 => report real time measurements, else report archived data run.
	unsigned char msbyteLswordDataRunId;//0 => report real time measurements, else report archived data run.
	unsigned char lsbyteMswordDataRunId;//0 => report real time measurements, else report archived data run.
	unsigned char msbyteMswordDataRunId;//0 => report real time measurements, else report archived data run.
	unsigned char lsbyteLswordMeasurementCount;//Number of measurements to report before stopping, ignored in real time case.
	unsigned char msbyteLswordMeasurementCount;
	unsigned char lsbyteMswordMeasurementCount;
	unsigned char msbyteMswordMeasurementCount;
	unsigned char lsbyteLswordMeasurementAddr;//Index of first measurement to report, , ignored in real time case.
	unsigned char msbyteLswordMeasurementAddr;
	unsigned char lsbyteMswordMeasurementAddr;
	unsigned char msbyteMswordMeasurementAddr;
} BlueIOStartMeasurementsParams; 

/***************************************************************************************************/
typedef struct
{
	unsigned char lsbyteLswordMeasurementPeriod;//Units are 'ticks'. Length of tick is different for each device type.
	unsigned char msbyteLswordMeasurementPeriod;
	unsigned char lsbyteMswordMeasurementPeriod;
	unsigned char msbyteMswordMeasurementPeriod;
} BlueIOSetMeasurementPeriodParams; //Parameter block passed into SendCmd() with BLUEIO_CMD_ID_SET_MEASUREMENT_PERIOD.

typedef BlueIOSetMeasurementPeriodParams BlueIOGetMeasurementPeriodCmdResponsePayload;//This is the response payload returned by GetNextResponse() 
																					//after sending BLUEIO_CMD_ID_GET_MEASUREMENT_PERIOD.

/***************************************************************************************************/
#define BLUEIO_LED_COLOR_BLACK 0xC0
#define BLUEIO_LED_COLOR_RED 0x40
#define BLUEIO_LED_COLOR_GREEN 0x80
#define BLUEIO_LED_COLOR_RED_GREEN 0
#define BLUEIO_LED_BRIGHTNESS_MIN 0
#define BLUEIO_LED_BRIGHTNESS_MAX 0x10

typedef struct
{
	unsigned char LEDchannel;//eg. WDSS_COLLECTION_LED
	unsigned char color;
	unsigned char brightness;
} BlueIOSetLedStateParams; //Parameter block passed into SendCmd() with BLUEIO_CMD_ID_SET_LED_STATE.

typedef struct
{
	unsigned char LEDchannel;
} BlueIOGetLedStateParams; //Parameter block passed into SendCmd() with BLUEIO_CMD_ID_GET_LED_STATE.

typedef BlueIOSetLedStateParams BlueIOGetLedStateCmdResponsePayload; //This is the response payload returned by GetNextResponse() after sending BLUEIO_CMD_ID_GET_LED_STATE.

/***************************************************************************************************/
#define BLUEIO_MASK_STATUS_ERROR_CMD_NOT_RECOGNIZED 1
#define BLUEIO_MASK_STATUS_ERROR_CMD_IGNORED 2
#define BLUEIO_MASK_STATUS_ERROR_MASTER_FIFO_OVERFLOW 0X40

//((system_status & BLUEIO_MASK_STATUS_BATTERY_STATE) == BLUEIO_MASK_STATUS_BATTERY_STATE_GOOD) => good batteries.
//((system_status & BLUEIO_MASK_STATUS_BATTERY_STATE) == BLUEIO_MASK_STATUS_BATTERY_STATE_LOW_WHILE_SAMPLING) => marginal batteries.
//((system_status & BLUEIO_MASK_STATUS_BATTERY_STATE) == BLUEIO_MASK_STATUS_BATTERY_STATE_LOW_ALWAYS) => bad batteries.
//((system_status & BLUEIO_MASK_STATUS_BATTERY_STATE) == BLUEIO_MASK_STATUS_BATTERY_STATE_MISSING) => no batteries.present.
#define BLUEIO_MASK_STATUS_BATTERY_STATE 0X0C
#define BLUEIO_MASK_STATUS_BATTERY_STATE_GOOD 0
#define BLUEIO_MASK_STATUS_BATTERY_STATE_LOW_WHILE_SAMPLING 4
#define BLUEIO_MASK_STATUS_BATTERY_STATE_LOW_ALWAYS 8
#define BLUEIO_MASK_STATUS_BATTERY_STATE_MISSING 0X0C

typedef struct
{
	unsigned char status;	//See BLUEIO_MASK_STATUS_*.
	unsigned char minorVersionMasterCPU;	//Binary coded decimal
	unsigned char majorVersionMasterCPU;	//Binary coded decimal
	unsigned char minorVersionSlaveCPU;		//Binary coded decimal - not used when target device has only one CPU
	unsigned char majorVersionSlaveCPU;		//Binary coded decimal - not used when target device has only one CPU
} BlueIOGetStatusCmdResponsePayload; //This is the response payload returned by GetNextResponse() after sending BLUEIO_CMD_ID_GET_STATUS.

/***************************************************************************************************/
typedef struct
{
	unsigned char ww;			//week in year(starting at 1) in BCD format
	unsigned char yy;			//last two digits of year in BCD format
	unsigned char lsbyteLswordSerialCounter;
	unsigned char msbyteLswordSerialCounter;
	unsigned char lsbyteMswordSerialCounter;
	unsigned char msbyteMswordSerialCounter;
} BlueIOGetSerialNumberCmdResponsePayload; //This is the response payload returned by GetNextResponse() after sending BLUEIO_CMD_ID_GET_SERIAL_NUMBER.

/***************************************************************************************************/
typedef struct
{
	unsigned char channel;	//(channel == -1) => memory local to interface.
	unsigned char lsbyteLswordAddr;
	unsigned char msbyteLswordAddr;
	unsigned char lsbyteMswordAddr;
	unsigned char msbyteMswordAddr;
	unsigned char count;
	unsigned char payload[1];	//The size of this payload varies. 
} BlueIOWriteNVMemParams; //Used with BLUEIO_CMD_ID_WRITE_NV_MEM.

typedef struct
{
	unsigned char channel;	//(channel == -1) => memory local to interface.
	unsigned char lsbyteLswordAddr;
	unsigned char msbyteLswordAddr;
	unsigned char lsbyteMswordAddr;
	unsigned char msbyteMswordAddr;
	unsigned char lsByteCount;		//Reads can span many blobs.
	unsigned char msByteCount;
} BlueIOReadNVMemParams; //Used with BLUEIO_CMD_ID_READ_NV_MEM.

typedef struct
{
	unsigned char payload[1];	//The size of this payload varies. 
} BlueIOReadNVMemCmdResponsePayload; //Used with BLUEIO_CMD_ID_READ_NV_MEM.
/***************************************************************************************************/
typedef struct
{
	unsigned char lsbyteLsword_EnableSensorChannels;//Enable bits for sensor channels 0 thru 7. Channel 0 is usually reserved for time.
	unsigned char msbyteLsword_EnableSensorChannels;//Enable bits for sensor channels 8 thru 15.
	unsigned char lsbyteMsword_EnableSensorChannels;//Enable bits for sensor channels 16 thru 23.
	unsigned char msbyteMsword_EnableSensorChannels;//Enable bits for sensor channels 24 thru 31.
} BlueIOSetSensorChannelEnableMaskParams; //Used with BLUEIO_CMD_ID_SET_SENSOR_CHANNEL_ENABLE_MASK.

//This is the response payload returned by GetNextResponse() after sending BLUEIO_CMD_ID_GET_SENSOR_CHANNEL_ENABLE_MASK.
typedef BlueIOSetSensorChannelEnableMaskParams BlueIOGetSensorChannelEnableMaskCmdResponsePayload;

/***************************************************************************************************/
//BLUEIO_CMD_ID_SET_COLLECTION_PARAMS:

#define BLUEIO_TRIGGER_MASK_BUTTON 1
#define BLUEIO_TRIGGER_MASK_CHANNEL 2

#define BLUEIO_TRIGGER_CHAN_MODE_MEAS_ABOVE_THRESHOLD 0
#define BLUEIO_TRIGGER_CHAN_MODE_MEAS_BELOW_THRESHOLD 1
#define BLUEIO_TRIGGER_CHAN_MODE_LO_TO_HI_MEAS 2
#define BLUEIO_TRIGGER_CHAN_MODE_HI_TO_LO_MEAS 3

#define BLUEIO_COLLECTION_FLAGS_MASK_IGNORE_STOP_BUTTON 1 // if (flags & 1) then the WDSS start/stop button is disabled during remote collection (to prevent accidental stop of remote collections)
#define BLUEIO_COLLECTION_FLAGS_MASK_REMOTE_TRIGGER_DELTA 2 // if (flags & 2) then remote trigger is a delta from the 1st collected value ==> e.g. with an altimeter (ver 1.08 and above)

typedef struct
{
	unsigned char lsbyteLswordDataRunId;//ffffffffh => next non RT data run, 0 => RT, else id's archived run.
	unsigned char msbyteLswordDataRunId;//ffffffffh => next non RT data run, 0 => RT, else id's archived run.
	unsigned char lsbyteMswordDataRunId;//ffffffffh => next non RT data run, 0 => RT, else id's archived run.
	unsigned char msbyteMswordDataRunId;//ffffffffh => next non RT data run, 0 => RT, else id's archived run.
	unsigned char triggerMask;		//BLUEIO_TRIGGER_MASK_BUTTON | BLUEIO_TRIGGER_MASK_CHANNEL
	unsigned char triggerChannelMode;//BLUEIO_TRIGGER_CHAN_MODE_...
	unsigned char triggerChannel;	//Used if BLUEIO_TRIGGER_MASK_CHANNEL is set in triggerMask.
	unsigned char flags;			//BLUEIO_COLLECTION_FLAGS_MASK_...
	unsigned char lsbyteTriggerThreshold;
	unsigned char msbyteTriggerThreshold;
	unsigned char lsbyteLswordMeasurementCount;//Number of measurements in a single data run.
	unsigned char msbyteLswordMeasurementCount;//Number of measurements in a single data run.
	unsigned char lsbyteMswordMeasurementCount;//Number of measurements in a single data run.
	unsigned char msbyteMswordMeasurementCount;//Number of measurements in a single data run.
	unsigned char lsbyteLswordPreTriggerMeasurementCount;//Number of pretrigger measurements in a data run.
	unsigned char msbyteLswordPreTriggerMeasurementCount;//Number of pretrigger measurements in a data run.
	unsigned char lsbyteMswordPreTriggerMeasurementCount;//Number of pretrigger measurements in a data run.
	unsigned char msbyteMswordPreTriggerMeasurementCount;//Number of pretrigger measurements in a data run.
	unsigned char lsbyteLsword_EnableSensorChannels;//Enable bits for sensor channels 0 thru 7. Channel 0 is usually reserved for time.
	unsigned char msbyteLsword_EnableSensorChannels;//Enable bits for sensor channels 8 thru 15.
	unsigned char lsbyteMsword_EnableSensorChannels;//Enable bits for sensor channels 16 thru 23.
	unsigned char msbyteMsword_EnableSensorChannels;//Enable bits for sensor channels 24 thru 31.
	unsigned char lsbyteLswordMeasurementPeriod;//Units are 'ticks'. Length of tick is different for each device type.
	unsigned char msbyteLswordMeasurementPeriod;
	unsigned char lsbyteMswordMeasurementPeriod;
	unsigned char msbyteMswordMeasurementPeriod;
	unsigned char hostDatum0;
	unsigned char hostDatum1;
	unsigned char hostDatum2;
	unsigned char hostDatum3;
} BlueIOSetCollectionParams; //Parameter block passed into SendCmd() with BLUEIO_CMD_ID_SET_COLLECTION_PARAMS.
//BLUEIO_CMD_ID_SET_COLLECTION_PARAMS can be used to setup both 'real time' and 'non real time' collections.
//A real time collection is triggered when the device receives BLUEIO_CMD_ID_START_MEASUREMENTS, and measurements
//are streamed up to the host computer into the BlueIO circular measurement buffers immediately. Real time collections
//continue indefinitely until the device receives BLUEIO_CMD_ID_STOP_MEASUREMENTS. The trigger and MeasurementCount
//parameters are ignored for real time runs. A DataRunId of 0 => real time.

//A DataRunId of 0xffffffff indicates that we are configuring the device for the next non real time collection.
//Non real time collections are also described as 'remote' collections. Remote collections are initiated by the user
//pressing the start button on the WDSS, and then the configured trigger conditions are applied. Up to 20 remote
//collections can be stored on the WDSS, and BLUEIO_CMD_ID_GET_COLLECTION_PARAMS is used to retrieve the collection
//params for the run identified by the specified DataRunId. BLUEIO_CMD_ID_GET_ARCHIVED_RUN_IDS is used to retrieve
//DataRunId's for the runs stored on the WDSS. The measurements associated with a remote run can be obtained by
//sending BLUEIO_CMD_ID_START_MEASUREMENTS with a BlueIOStartMeasurementsParams block identifying the DataRunId.
//Thus both real time and non real time measurements can be retrieved by calling BlueIO_Device_ReadRawMeasurements().

//Note that real time collections can be setup without using BLUEIO_CMD_ID_SET_COLLECTION_PARAMS by using
//BlueIO_Device_SetMeasurementPeriod() and BLUEIO_CMD_ID_SET_SENSOR_CHANNEL_ENABLE_MASK instead.

typedef struct
{
	unsigned char lsbyteLswordDataRunId;//ffffffffh => next non RT data run, 0 => RT, else id's archived run.
	unsigned char msbyteLswordDataRunId;//ffffffffh => next non RT data run, 0 => RT, else id's archived run.
	unsigned char lsbyteMswordDataRunId;//ffffffffh => next non RT data run, 0 => RT, else id's archived run.
	unsigned char msbyteMswordDataRunId;//ffffffffh => next non RT data run, 0 => RT, else id's archived run.
} BlueIOGetCollectionParams; //Parameter block passed into SendCmd() with BLUEIO_CMD_ID_GET_COLLECTION_PARAMS.

typedef BlueIOSetCollectionParams BlueIOGetCollectionParamsCmdResponsePayload;//This is the response payload returned by GetNextResponse() 
																	//after sending BLUEIO_CMD_ID_GET_COLLECTION_PARAMS.
/***************************************************************************************************/
//BLUEIO_CMD_ID_GET_ARCHIVED_RUN_IDS:

typedef struct
{
	unsigned char firstRunIndex;	//Retrieve id's for firstRunIndex'th run thru the (firstRunIndex + numRuns -1 )'th run.
	unsigned char numRuns;
} BlueIOGetArchivedRunIdsParams;

typedef struct
{
	unsigned char numRuns;		//Can never exceed BlueIOGetArchivedRunIdsParams.numRuns. 
	unsigned char lsbyteLswordDataRunIdA;
	unsigned char msbyteLswordDataRunIdA;
	unsigned char lsbyteMswordDataRunIdA;
	unsigned char msbyteMswordDataRunIdA;
//	unsigned char lsbyteLswordDataRunIdB;
//	unsigned char msbyteLswordDataRunIdB;
//	unsigned char lsbyteMswordDataRunIdB;
//	unsigned char msbyteMswordDataRunIdB;
//   ...
} BlueIOGetArchivedRunIdsCmdResponsePayload;

/***************************************************************************************************/
//BLUEIO_CMD_ID_DELETE_ARCHIVED_RUNS:

typedef BlueIOGetArchivedRunIdsCmdResponsePayload BlueIODeleteArchivedRunsParams;
//(BlueIODeleteArchivedRunsParams.numRuns == 0xff) => delete all archived runs.

/***************************************************************************************************/

// Restore the old packing scheme
// These commands are in a few places throughout the code so if youchange one, change them all!
#if defined TARGET_OS_WIN
	#pragma pack(pop)
#elif (defined TARGET_OS_MAC) || (defined TARGET_OS_LINUX)
	#pragma pack()
#endif

#endif 
