// BlueIO_MeasureDoc.cpp : implementation of the CBlueIO_MeasureDoc class
//

#include "stdafx.h"
#include "BlueIO_Measure.h"

#include "MainFrm.h"
#include "BlueIO_MeasureDoc.h"
#include "BlueIO_MeasureView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CBlueIO_MeasureDoc

IMPLEMENT_DYNCREATE(CBlueIO_MeasureDoc, CDocument)

BEGIN_MESSAGE_MAP(CBlueIO_MeasureDoc, CDocument)
END_MESSAGE_MAP()


// CBlueIO_MeasureDoc construction/destruction

CBlueIO_MeasureDoc::CBlueIO_MeasureDoc()
{
	// TODO: add one-time construction code here
	m_pDevice = NULL;
	m_activeChannel = 99;
	m_pMeasCirBuf = new GCircularBuffer(MAX_NUM_MEASUREMENTS_IN_CIRBUF*sizeof(double));
	m_pTimeCirBuf = new GCircularBuffer(MAX_NUM_MEASUREMENTS_IN_CIRBUF*sizeof(double));

	ClearMeasurementCirbuf();
	m_measPeriodInSeconds = 0.5;
}

CBlueIO_MeasureDoc::~CBlueIO_MeasureDoc()
{
	if (m_pDevice)
		CloseDevice();

	if (m_pTimeCirBuf)
		delete m_pTimeCirBuf;
	m_pTimeCirBuf = NULL;

	if (m_pMeasCirBuf)
		delete m_pMeasCirBuf;
	m_pMeasCirBuf = NULL;
}

BLUEIO_DEVICE_HANDLE CBlueIO_MeasureDoc::OpenDevice(LPCSTR pDeviceName)
{
	if (m_pDevice)
		CloseDevice();
	m_pDevice = BlueIO_Device_Open(hBlueIO_lib, pDeviceName);
	if (m_pDevice)
	{
		SetActiveChannel(WDSS_CHANNEL_COMBINED_FORCE);
		m_measPeriodInSeconds = 0.01;
		BlueIO_Device_SetMeasurementPeriod(m_pDevice, m_measPeriodInSeconds, BLUEIO_TIMEOUT_MS_DEFAULT);
	}
	return m_pDevice;
}

void CBlueIO_MeasureDoc::CloseDevice()
{
	if (m_pDevice)
		BlueIO_Device_Close(m_pDevice);
	m_pDevice = NULL;

	ClearMeasurementCirbuf();

	CMainFrame *pFrame = (CMainFrame *) AfxGetMainWnd();
	if (pFrame)
	{
		SetTitle(L"");
		pFrame->m_wndCollectBar.SetDlgItemText(IDC_MEAS, L"");
		pFrame->m_wndCollectBar.SetDlgItemText(IDC_UNITS, L"");
		CBlueIO_MeasureView *pView = (CBlueIO_MeasureView *) (pFrame->GetActiveView());
		if (pView)
		{
			pView->SetGraphHistory(1.0, -1.0);//This clears the history.
			pView->Invalidate();
		}
	}

	SetTitle(L"");
}

BOOL CBlueIO_MeasureDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
	ClearMeasurementCirbuf();
	CFrameWnd *pFrame = (CFrameWnd *) AfxGetMainWnd();
	if (pFrame)
	{
		CBlueIO_MeasureView *pView = (CBlueIO_MeasureView *) pFrame->GetActiveView();
		if (pView)
			pView->SetGraphHistory(1.0, -1.0);//This clears the history.

		int oldChannel = GetActiveChannel();
		SetActiveChannel(0);
		SetActiveChannel(oldChannel);
	}

	return TRUE;
}




// CBlueIO_MeasureDoc serialization

void CBlueIO_MeasureDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

void CBlueIO_MeasureDoc::SetActiveChannel(unsigned char channel)
{ 
	if (m_activeChannel != channel)
	{
		m_activeChannel = channel;
		if (m_pDevice)
		{
			unsigned char mask;
			switch (channel)
			{
				case WDSS_CHANNEL_BAROMETER:
					mask = WDSS_CHANNEL_MASK_BAROMETER;
					break;
				case WDSS_CHANNEL_ACCEL_Y:
					mask = WDSS_CHANNEL_MASK_ACCEL_Y;
					break;
				case WDSS_CHANNEL_ACCEL_X:
					mask = WDSS_CHANNEL_MASK_ACCEL_X;
					break;
				case WDSS_CHANNEL_ACCEL_Z:
					mask = WDSS_CHANNEL_MASK_ACCEL_Z;
					break;
				default:
					mask = WDSS_CHANNEL_MASK_COMBINED_FORCE;
					m_activeChannel = WDSS_CHANNEL_COMBINED_FORCE;
					break;
			}
			BlueIOSetSensorChannelEnableMaskParams params;
			memset(&params, 0, sizeof(params));
			params.lsbyteLsword_EnableSensorChannels = mask;
			BlueIO_Device_SendCmdAndGetResponse(m_pDevice, BLUEIO_CMD_ID_SET_SENSOR_CHANNEL_ENABLE_MASK, 
				&params, sizeof(params), NULL, NULL, BLUEIO_TIMEOUT_MS_DEFAULT);

			unsigned char activeCalPageIndex;
			gtype_real32 a, b, c;
			char units[10];
			WCHAR wUnits[10];
			BlueIO_Device_DDSMem_GetActiveCalPage(m_pDevice, m_activeChannel, &activeCalPageIndex);
			BlueIO_Device_DDSMem_GetCalPage(m_pDevice, m_activeChannel, activeCalPageIndex,
				&a, &b, &c, units, sizeof(units));
			MultiByteToWideChar(CP_ACP, 0, units, -1, wUnits, 10);

			ClearMeasurementCirbuf();

			CMainFrame *pFrame = (CMainFrame *) AfxGetMainWnd();
			if (pFrame)
			{
				WCHAR title[100];
				char sensorName[50];
				WCHAR wSensorName[50];
				BlueIO_Device_DDSMem_GetLongName(m_pDevice, m_activeChannel, sensorName, sizeof(sensorName));
				MultiByteToWideChar(CP_ACP, 0, sensorName, -1, wSensorName, 50);
				wsprintf(title, L"WDSS %s Sensor", wSensorName);
				SetTitle(title);

				pFrame->m_wndCollectBar.SetDlgItemText(IDC_MEAS, L"");
				pFrame->m_wndCollectBar.SetDlgItemText(IDC_UNITS, wUnits);
				CBlueIO_MeasureView *pView = (CBlueIO_MeasureView *) (pFrame->GetActiveView());
				if (pView)
				{
					pView->SetGraphHistory(1.0, -1.0);//This clears the history.
					pView->Invalidate();
				}
			}
		}
	}
}

// CBlueIO_MeasureDoc diagnostics

#ifdef _DEBUG
void CBlueIO_MeasureDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CBlueIO_MeasureDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG



void CBlueIO_MeasureDoc::AddMeasurementToCirbuf(double measurement, double time)
{
	m_pMeasCirBuf->AddBytes((unsigned char *) &measurement, sizeof(measurement));
	m_pTimeCirBuf->AddBytes((unsigned char *) &time, sizeof(time));
}

void CBlueIO_MeasureDoc::ClearMeasurementCirbuf()
{
	m_pMeasCirBuf->Clear();
	m_pTimeCirBuf->Clear();
}

bool CBlueIO_MeasureDoc::GetNthMeasurementInCirbuf(int N, double *pMeasurement, double *pTime) //(N == 0) => first measurement.
{
	bool bSuccess = false;
	double measurement = 0.0;
	double time = 0.0;
	int count = m_pMeasCirBuf->CopyBytes((unsigned char *) &measurement, N*sizeof(double), sizeof(double));
	if (sizeof(double) == count)
	{
		count = m_pTimeCirBuf->CopyBytes((unsigned char *) &time, N*sizeof(double), sizeof(double));
		if (sizeof(double) == count)
		{
			bSuccess = true;
			*pMeasurement = measurement;
			*pTime = time;
		}
	}

	return bSuccess;
}

