// BlueIO_MeasureDoc.h : interface of the CBlueIO_MeasureDoc class
//

#pragma once

#include "BlueIO_lib_interface.h"
#include "GCircularBuffer.h"

#define MAX_NUM_MEASUREMENTS_IN_CIRBUF 1001

class CBlueIO_MeasureDoc : public CDocument
{
protected: // create from serialization only
	CBlueIO_MeasureDoc();
	DECLARE_DYNCREATE(CBlueIO_MeasureDoc)

// Attributes
public:
	BLUEIO_DEVICE_HANDLE GetOpenDevicePtr() { return m_pDevice; }

// Operations
public:
	BLUEIO_DEVICE_HANDLE OpenDevice(LPCSTR pDeviceName);
	void CloseDevice();

	unsigned char GetActiveChannel() 
	{ 
		if (m_activeChannel > WDSS_CHANNEL_COMBINED_FORCE)
			return WDSS_CHANNEL_COMBINED_FORCE;
		else
			return m_activeChannel; 
	}
	void SetActiveChannel(unsigned char channel);
	void AddMeasurementToCirbuf(double measurement, double time);
	int GetNumMeasurementsInCirbuf() { return m_pMeasCirBuf->NumBytesAvailable()/sizeof(double); }
	void ClearMeasurementCirbuf();
	bool GetNthMeasurementInCirbuf(int N, double *pMeasurement, double *pTime); //(N == 0) => first measurement.
	void SetMeasurementPeriodInSeconds(double period) { m_measPeriodInSeconds = period; }
	double GetMeasurementPeriodInSeconds() { return m_measPeriodInSeconds; }

// Overrides
public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);

// Implementation
public:
	virtual ~CBlueIO_MeasureDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	BLUEIO_DEVICE_HANDLE m_pDevice;
	double m_measPeriodInSeconds;
	GCircularBuffer *m_pMeasCirBuf;
	GCircularBuffer *m_pTimeCirBuf;
	unsigned char m_activeChannel;

// Generated message map functions
protected:
	DECLARE_MESSAGE_MAP()
};


