// BlueIO_MeasureView.cpp : implementation of the CBlueIO_MeasureView class
//

#include "stdafx.h"
#include <math.h>
#include "BlueIO_Measure.h"

#include "BlueIO_MeasureDoc.h"
#include "BlueIO_MeasureView.h"

#include <sstream>
#include <iomanip>

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CBlueIO_MeasureView

IMPLEMENT_DYNCREATE(CBlueIO_MeasureView, CView)

BEGIN_MESSAGE_MAP(CBlueIO_MeasureView, CView)
END_MESSAGE_MAP()

// CBlueIO_MeasureView construction/destruction

CBlueIO_MeasureView::CBlueIO_MeasureView()
{
	graph_history_y_min = 1.0;	//Invalid at first.
	graph_history_y_max = -1.0;	//Invalid at first.
}

CBlueIO_MeasureView::~CBlueIO_MeasureView()
{
}

BOOL CBlueIO_MeasureView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CBlueIO_MeasureView drawing

void CBlueIO_MeasureView::OnDraw(CDC* pDC)
{
	CBlueIO_MeasureDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	WCHAR tmpstring[100];

	CRect clientRect;
	GetClientRect(&clientRect);

	BLUEIO_DEVICE_HANDLE hDevice = pDoc->GetOpenDevicePtr();
	if (hDevice && (pDoc->GetNumMeasurementsInCirbuf() > 0))
	{
		CRect graphRect = clientRect;
		int i;
		double meas_x_range, meas_x_min, meas_x_max;
		double meas_y_range, meas_y_min, meas_y_max, testy, testx;
		int x, y, deltaX, deltaY;

		//Calculate what portion of the client area will hold the graph.
		if ((graphRect.Width() > 200) && (graphRect.Height() > 200))
		{
			graphRect.top += 25;
			graphRect.bottom -= 50;
			graphRect.left += 100;
			graphRect.right -= 25;
		}

		//Calculate full graph time range.
		int numMeasurements = pDoc->GetNumMeasurementsInCirbuf();
		meas_x_min = 0.0;
		meas_x_max = 1.0;
		if (numMeasurements > 0)
		{
			pDoc->GetNthMeasurementInCirbuf(0, &testy, &meas_x_min);
			pDoc->GetNthMeasurementInCirbuf(numMeasurements - 1, &testy, &meas_x_max);
		}
		meas_x_range = meas_x_max - meas_x_min;

		//Calculate full graph y range.
		pDoc->GetNthMeasurementInCirbuf(0, &meas_y_min, &testx);
		meas_y_max = meas_y_min;
		for (i = 1; i < numMeasurements; i++)
		{
			pDoc->GetNthMeasurementInCirbuf(i, &testy, &testx);
			if (testy < meas_y_min)
				meas_y_min = testy;
			if (testy > meas_y_max)
				meas_y_max = testy;
		}
		meas_y_range = meas_y_max - meas_y_min;

		//Make sure that meas_y_range corresponds to a voltage delta of at least 0.1 volts.
		if (meas_y_range < 0.1)
		{
			meas_y_range = 0.1;
			meas_y_max = meas_y_min + meas_y_range;
		}
/*
		double y_0_volts, y_dot1_volts, deltay_dot1_volts;
		y_0_volts = GoIO_Sensor_CalibrateData(hDevice, 0.0);
		y_dot1_volts = GoIO_Sensor_CalibrateData(hDevice, 0.1);
		deltay_dot1_volts = y_dot1_volts - y_0_volts;
		if (deltay_dot1_volts < 0.0)
			deltay_dot1_volts = -deltay_dot1_volts;
		if (deltay_dot1_volts > meas_y_range)
		{
			meas_y_range = deltay_dot1_volts;
			meas_y_max = meas_y_min + meas_y_range;
		}
*/
		double history_y_min;
		double history_y_max;
		GetGraphHistory(history_y_min, history_y_max);
		if (history_y_max > history_y_min)
		{
			if (history_y_max > meas_y_max)
				meas_y_max = history_y_max;
			if (history_y_min < meas_y_min)
				meas_y_min = history_y_min;
			meas_y_range = meas_y_max - meas_y_min;
		}
		SetGraphHistory(meas_y_min, meas_y_max);

		//Do some drawing now.
		pDC->FillRect(&clientRect, CBrush::FromHandle((HBRUSH) ::GetStockObject(WHITE_BRUSH)));

		CPen *pOldPen = pDC->SelectObject(CPen::FromHandle((HPEN) ::GetStockObject(BLACK_PEN)));

		pDC->MoveTo(graphRect.left, clientRect.top);
		pDC->LineTo(graphRect.left, clientRect.bottom);
		pDC->MoveTo(graphRect.right, clientRect.top);
		pDC->LineTo(graphRect.right, clientRect.bottom);

		CPen bluePen;
		bluePen.CreatePen(PS_SOLID, 0, RGB(0, 0, 200));
		pDC->SelectObject(&bluePen);

		pDC->MoveTo(clientRect.left, graphRect.bottom);
		pDC->LineTo(clientRect.right, graphRect.bottom);
		pDC->MoveTo(clientRect.left, graphRect.top);
		pDC->LineTo(clientRect.right, graphRect.top);

		std::wstringstream ss;
		ss << fixed << showpoint << setprecision(3) << meas_x_min << L" secs";
		std::wstring str = ss.str();
		lstrcpy(tmpstring, str.c_str());
		x = graphRect.left + 10;
		y = graphRect.bottom + 10;
		pDC->TextOut(x, y, tmpstring, lstrlen(tmpstring));

		ss.str(L"");
		ss << fixed << showpoint << setprecision(3) << meas_x_max << L" secs";
		str = ss.str();
		lstrcpy(tmpstring, str.c_str());
		CRect textRect;
		pDC->DrawText(tmpstring, lstrlen(tmpstring), &textRect, DT_CALCRECT);
		x = graphRect.right - textRect.Width() - 10;
		y = graphRect.bottom + 10;
		pDC->TextOut(x, y, tmpstring, lstrlen(tmpstring));

		unsigned char activeCalPageIndex;
		gtype_real32 a, b, c;
		char units[10];
		BlueIO_Device_DDSMem_GetActiveCalPage(hDevice, pDoc->GetActiveChannel(), &activeCalPageIndex);
		BlueIO_Device_DDSMem_GetCalPage(hDevice, pDoc->GetActiveChannel(), activeCalPageIndex,
			&a, &b, &c, units, sizeof(units));

		pDC->SetTextColor(RGB(0, 0, 200));
		pDC->SetBkMode(TRANSPARENT);

		ss.str(L"");
		ss << fixed << showpoint << setprecision(3) << meas_y_min << " " << units;
		str = ss.str();
		lstrcpy(tmpstring, str.c_str());
		pDC->DrawText(tmpstring, lstrlen(tmpstring), &textRect, DT_CALCRECT);
		x = clientRect.left + 10;
		y = graphRect.bottom - textRect.Height() - 10;
		pDC->TextOut(x, y, tmpstring, lstrlen(tmpstring));

		ss.str(L"");
		ss << fixed << showpoint << setprecision(3) << meas_y_max << " " << units;
		str = ss.str();
		lstrcpy(tmpstring, str.c_str());
		x = clientRect.left + 10;
		y = graphRect.top + 10;
		pDC->TextOut(x, y, tmpstring, lstrlen(tmpstring));


		CPen redPen;
		redPen.CreatePen(PS_SOLID, 3, RGB(200, 0, 0));
		pDC->SelectObject(&redPen);

		double meas_x_frac, meas_y_frac;

		for (i = 0; i < numMeasurements; i++)
		{
			pDoc->GetNthMeasurementInCirbuf(i, &meas_y_frac, &meas_x_frac);
			meas_y_frac = (meas_y_frac - meas_y_min)/meas_y_range;
			deltaY = (int) floor(meas_y_frac*graphRect.Height() + 0.5);
			meas_x_frac = (meas_x_frac - meas_x_min)/meas_x_range;
			deltaX = (int) floor(meas_x_frac*graphRect.Width() + 0.5);
			x = graphRect.left + deltaX;
			y = graphRect.bottom - deltaY;
			if (0 == i)
				pDC->MoveTo(x, y);
			pDC->LineTo(x, y);
		}

		if (pOldPen)
			pDC->SelectObject(pOldPen);
	}
	else
		pDC->FillRect(&clientRect, CBrush::FromHandle((HBRUSH) ::GetStockObject(WHITE_BRUSH)));
}

// CBlueIO_MeasureView diagnostics

#ifdef _DEBUG
void CBlueIO_MeasureView::AssertValid() const
{
	CView::AssertValid();
}

void CBlueIO_MeasureView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CBlueIO_MeasureDoc* CBlueIO_MeasureView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBlueIO_MeasureDoc)));
	return (CBlueIO_MeasureDoc*)m_pDocument;
}
#endif //_DEBUG


// CBlueIO_MeasureView message handlers
